/*
 * Copyright (c) 2006-2009 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef __GPS_STRUCTURE_OEM4
#define __GPS_STRUCTURE_OEM4



/////////////////////////////////////////////////////////////////
//
// Header d'un trame binaire recu du GPS
//
/////////////////////////////////////////////////////////////////
typedef struct  OEM4_BINARY_HEADER       // Standard binary header
{
  unsigned char          sop1;          // start of packet first byte
  unsigned char          sop2;          // start of packet second byte
  unsigned char          sop3;          // start of packet third  byte
  unsigned char          header_length; // Length of the header
  unsigned short         number;        // Message number
  unsigned char          type;          // Message type
  unsigned char          port_address;  // Address of the data port the log was received on
  unsigned short         length;        // Message length
  unsigned short         sequence;      // Sequence #
  unsigned char          idle;          // Idle time
  unsigned char          gps_stat;      // GPS Time Status 
  unsigned short         gps_week;      // GPS Week number
  unsigned int           millisecs;     // Milliseconds into week
  unsigned int           status;        // Receiver status word
  unsigned short         internalUse;   // Internal use
  unsigned short         version;       // Receiver software version
  unsigned short         crc;           // The 16bit CRC check sum
} OEM4_BINARY_HEADER;

/////////////////////////////////////////////////////////////////
  //
  // BINARY BEST POSITION MODE RTK
  //
  /////////////////////////////////////////////////////////////////
typedef struct OEM4BESTPOSb_LOG   
{
  unsigned int   sol_stat;           // Solution status
  unsigned int   pos_type;           // Position type
  double         lat;                // Latitude
  double         lon;                // Longitude
  double         hgt;                // Height above mean sea level
  float          und;                // Undulation
  long           datum_id;           // Datum ID
  float          latdev;             // Latitude standard deviation
  float          londev;             // Longitude standard deviation
  float          hgtdev;             // Height standard deviation
  char           ref_station_id[4];  // Reference station ID
  float          lag;                // Differential correction lag
  float          age;                // Solution age
  unsigned char  sats;               // Number of matched satellites
  unsigned char  gps_l1_ranges;      // Number of GPS L1 ranges used
  unsigned char  gps_l1_mask_ranges; // Number of GPS L1 ranges used
  unsigned char  gps_l2_mask_ranges; // Number of GPS L2 ranges used
  char           reserved[4];
} OEM4BESTPOSb_LOG;


// BESTPOS log structure
typedef struct  BESTPOSb_LOG
{
  OEM4_BINARY_HEADER      hdr;
  OEM4BESTPOSb_LOG            data;
} BESTPOSb_LOG;

/////////////////////////////////////////////////////////////////
  //
  // BINARY BESTUTM MODE RTK
  //
  /////////////////////////////////////////////////////////////////
typedef struct  OEM4BESTUTMb_LOG   
{
  unsigned int   solStatus;             // Solution status
  unsigned int   posType;               // Position type
  unsigned int   longitudZoneNumber;    // Longitudinal zone number
  unsigned int   latitudinalZoneLetter; // Latitudinal zone letter
  double         northing;              // Northing origin
  double         easting;               // Easting origin
  double         height;                // Height above mean sea level
  float          undulation;            // Undulation 
  unsigned int   datumID;               // Datum ID number
  float          northingDev;           // Northing st. deviation
  float          eastingDev;            // Easting st. deviation
  float          heightDev;             // Height st. deviation
  char           baseStationID[4];      // Base station ID 
  float          differentialAge;       // Differential age in seconds
  float          solutionAge;           // Solution age in seconds
  unsigned char  nbSatTracked;          // Number of satellite tracked
  unsigned char  nbGPSL1Used;           // Number of GPS L1 ranges used in computation
  unsigned char  nbGPSL1;               // Number of GPS L1 ranges above RTK mask angle
  unsigned char  nbGPSL2;               // Number of GPS L2 ranges above RTK mask angle
  char           reserved[4];           // reserved
} OEM4BESTUTMb_LOG;


// BESTUTM log structure
typedef struct BESTUTMb_LOG
{
  OEM4_BINARY_HEADER      hdr;
  OEM4BESTUTMb_LOG        data;
} BESTUTMb_LOG;

/////////////////////////////////////////////////////////////////
  //
  // BINARY BESTVEL MODE RTK
  //
  /////////////////////////////////////////////////////////////////
typedef struct OEM4BESTVELb_LOG    
{
  unsigned int   solStatus;             // Solution status
  unsigned int   velocityType;          // Velocity type
  float          latency;               // Latency
  float          age;                   // Differential age in seconds
  double         horizontalSpeed;       // Horizontal speed over ground
  double         actualDirection;       // Actual direction of motion
  double         verticalSpeed;         // Vertical speed in m/s, +->up, --> down
  float          reserved;              // Bon cadeau regally
} OEM4BESTVELb_LOG;


// BESTUTM log structure
typedef struct BESTVELb_LOG
{
  OEM4_BINARY_HEADER      hdr;
  OEM4BESTVELb_LOG        data;
} BESTVELb_LOG;


/////////////////////////////////////////////////////////////////
  //
  // BINARY BESTXYZ MODE RTK
  //
  /////////////////////////////////////////////////////////////////
typedef struct OEM4BESTXYZb_LOG 
{
  unsigned int   solStatus;                   // Solution status
  unsigned int   positionType;                // Position type
  double         positionX;                   // Position X
  double         positionY;                   // Position Y
  double         positionZ;                   // Position Z
  float          stdPositionX;                // Standard deviation pos. X
  float          stdPositionY;                // Standard deviation pos. Y
  float          stdPositionZ;                // Standard deviation pos. Z
  unsigned int   vSolStatus;                  // Solution status 
  unsigned int   velStatus;                   // Velocity status
  double         velocityVectorX;             // Velocity vector X
  double         velocityVectorY;             // Velocity vector Y
  double         velocityVectorZ;             // Velocity vector Z
  float          stdDevVelocityVectorX;       // std. deviation Velocity vector X
  float          stdDevVelocityVectorY;       // std. deviation Velocity vector Y
  float          stdDevVelocityVectorZ;       // std. deviation Velocity vector Z
  char           baseID[4];                   // Base ID
  float          velocityLatency;             // Velocity latency
  float          differentialAge;             // Differential age in s
  float          solutionAge;                 // Solution age in s
  unsigned char  nbObsTracked;                // Number of observations tracked
  unsigned char  nbGPSL1Used;                 // Nb GPS L1 used in computation
  unsigned char  nbGPSL1RTK;                  // Nb of GPS L1 range above RTK mask angle
  unsigned char  nbGPSL2RTK;                  // Nb of GPS L2 range above RTK mask angle
  char           reserved[4];

} OEM4BESTXYZb_LOG;


// BESTUTM log structure
typedef struct BESTXYZb_LOG
{
  OEM4_BINARY_HEADER      hdr;
  OEM4BESTXYZb_LOG        data;
} BESTXYZb_LOG;


#endif

