/*
 * Copyright (c) 2006 Frederic Camps
 * Copyright (c) 2006-2009 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/*!
 *
 * $Source:
 * $Revision: 0.a
 * $Date: 24 april 2006
 * 
 */
#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <GPSNovatel/sampleAPI.h>
#include <GPSNovatel/serial.h>
#include <GPSNovatel/logData.h>

/*!
  @brief
 Utilisation du programme principal
*/
static void 
usage(const char *progname)
{
	fprintf(stderr, "usage: %s [-v] serial_device format={gpx,utm} [-l logFile] [-n max_trame]\n", progname);
	exit(1);
}

static const int DEFAULT_COM1_SPEED=57600;
static const int DEFAULT_COM2_SPEED=57600;

/*!
 @brief
 Main du programme principal
 */
int 
main(int argc, char *argv[])
{
	int fd, ch;
	char *prog = argv[0];
	
	int nbTrame=0;
	int max_trame = -1;
	
	static char *logFile=NULL;
	static int modeVerbose=0;
	static GPS_LOG_FORMAT format=GPS_LOG_UTM;
	
	while ((ch = getopt(argc, argv, "vl:n:")) != -1)	{
		switch (ch) {
		case 'v':
			modeVerbose=1;
			break;			
		case 'l':
			logFile = optarg;
			modeVerbose=2;
			break;
		case 'n':
			max_trame = atoi(optarg);
			break;

		case '?':
		case 'h':
		default:
			usage(prog);
		} /* switch */
	}
	
	argc -= optind;
	argv += optind;
	
	if (argc < 1)
		usage(prog);

	// Ouverture du port serie en mode non bloquant avec une
	// attente de 400ms et pas de contrainte sur la quantité de
	// données à lire
	if( (fd = initGPS(argv[0], 0, 4, DEFAULT_COM1_SPEED)) == -1)
		err(1, "Can't open %s", argv[0]);

	if (argv[1]!=NULL) {
	  if (!strcmp(argv[1],"utm"))
	    format=GPS_LOG_UTM;
	  else if (!strcmp(argv[1],"gpx"))
	    format=GPS_LOG_GPX;
	} else
	  format=GPS_LOG_UTM;
	

	// Initialise le mode verbeux et/ou des logs dans un fichier
	GPSNovatel_setLogData(NOV_TRUE, modeVerbose, logFile, format);
	
	// Initialisation du GPS
	//
	// Config des COM
	
	/* Set speed for com2 : corrections */
	char CONF_COM2[]="COM COM2 57600\n";
	writeSerial(fd,  CONF_COM2);
	
	/* set com2 to receive differential corrections */
	char INTERFACEMODE_COM2[]="INTERFACEMODE COM2 RTCMNOCR NONE OFF\n";
	writeSerial(fd, INTERFACEMODE_COM2);
	
	// Configuration des LOGS	
	char UNLOG_ALL_COM1[]="UNLOGALL COM1 TRUE\n"; // COM2 recoit les log de la base
	writeSerial(fd,  UNLOG_ALL_COM1);
	
	//  char LOG_BESTXYZb_COM1[]="LOG COM1 BESTXYZb ontime 1\n";
	//  writeSerial(fd, LOG_BESTXYZb_COM1);
	
	char LOG_BESTPOSb_COM1[]="LOG COM1 BESTPOSb ontime 1\n";
	writeSerial(fd, LOG_BESTPOSb_COM1);
	
	/* log info every 500ms */
	char LOG_BESTUTMb_COM1[]="LOG COM1 BESTUTMb ontime 0.5\n";
	writeSerial(fd, LOG_BESTUTMb_COM1);

	/* log speed every 500ms */
	char LOG_BESTVELb_COM1[]="LOG COM1 BESTVELb ontime 0.5\n";
	writeSerial(fd, LOG_BESTVELb_COM1);

	// read info
	// Lecture port serie
	while(max_trame != 0) {
	  nbTrame=0;
	  /* read data */
	  readSerial(fd, &nbTrame);

	  if (max_trame != -1 && nbTrame != -1) {
		  max_trame -= nbTrame;
		  if (max_trame < 0) max_trame = 0;
	  }
	  
	  /* log to the log file with gpx format */ 
	  if (nbTrame > 0)
		  GPSNovatel_logData();
	  
	}
	exit(0);
}
